setwd (dir = '/home/allgoodguys/Documents/Studying/Lund_PhD/experiments/focal-terms/analysis')
# set the language to work with
lang = c('sv','en','ru')[1]
exclude_response_times_over_60_sec = T  # set to T for response times, F for everything else

folder = paste0("/home/allgoodguys/Documents/Studying/Lund_PhD/experiments/focal-terms/output_",lang)
filenames <- list.files(folder, pattern="*.csv", full.names=TRUE)  # open all .csv files in folder / output_sv, _en, _ru
if (lang!='ru') {
  int <- lapply(filenames, read.csv, header=T, stringsAsFactors=FALSE)  # make a list of dataframes
} else {int <- lapply(filenames, read.csv, header=F, stringsAsFactors=FALSE)}
df = do.call(rbind, int)
if (lang=='ru') {
  df = df[3:ncol(df)]
  colnames(df) = c('id','order','lang','sex','sound','nameSound','sureSound','nameEmotion','sureEmotion','reps','time','timeSound','timeEmotion')
}

df$nameSound=as.character(df$nameSound)
df$nameSound = ifelse(df$nameSound=='', 'callDK',df$nameSound)
df$nameSound = as.factor(df$nameSound)

df$nameEmotion=as.character(df$nameEmotion)
df$nameEmotion = ifelse(df$nameEmotion=='', 'emDK',df$nameEmotion)
df$nameEmotion = as.factor(df$nameEmotion)

write.csv(df, "temp.csv"); df=read.csv('temp.csv')[,-1]

# set the names for calls and ems
if (lang=='sv'){
  calls = c('flämtning','fniss','gnäll','gråt','rop','skratt','skrik','snyftning','stön','suck','tjut','vrål','callDK')
  ems = c('ansträngning','frustration','glädje','hysteri','ilska','irritation','jubel','lättnad','munterhet','njutning','panik','rädsla','smärta','sorg','äckel','överraskning','emDK')
} else if (lang=='en') {
  calls = c('chuckle','cry','gasp','giggle','groan','growl','grunt','laugh','moan','roar','scream','shriek','sigh','snort','sob','yell','callDK')
  ems = c('amusement','anger','annoyance','disgust','effort','excitement','fear','fright','frustration','happiness','hysteria','pain','pleasure','relief','sadness','surprise','emDK')
} else if (lang=='ru'){
  calls = c('вздох','визг','вой','вопль','крик','кряхтенье','нытье','плач','рев','смех','стон','хихиканье','callDK')
  ems = c('боль','веселье','восторг','грусть','досада','злость','испуг','истерика','облегчение','отвращение','радость','раздражение','страх','удивление','удовольствие','усилие','emDK')
}


### convert df to something like in the rating experiment (1/0 for each call & emotion for each sound)
df1 = subset (df, select=c('id','order','lang','sex','sound','sureSound','sureEmotion','reps','time','timeSound','timeEmotion'))
for (i in c(calls,ems)) {  
  df1[,i] = 0
}
for (i in 1:nrow(df1)){
  temp = c ( unlist (strsplit(as.character(df$nameSound[i]), '-')), unlist (strsplit(as.character(df$nameEmotion[i]), '-')) )
  for (j in temp) {
    df1[i, j] = 1
  }
}
df1$topCall = apply (df1[, 12:(length(calls)+12-1)], 1, function(x) names(which.max(x)))
df1$topEm = apply (df1[, (length(calls)+12):(length(calls)+12+length(ems)-1) ], 1, function(x) names(which.max(x)))
# df1 = subset (df1, select = -c(callDK,emDK)) # without DK's
df1 = droplevels (df1[!df1$id %in% names(table(df$id)[table(df$id)<10]), ]) # drop id's with <10 responses
if (exclude_response_times_over_60_sec) {
  df1 = df1[df1$time<60000,]
}

# write.csv(df1, 'df1_ru_for-time.csv')
write.csv(df1, '~/Downloads/temp.csv'); df1 = read.csv('~/Downloads/temp.csv')[,-1]

table_simpl = df1[1:132, 5:ncol(df1)]
colnames(table_simpl)  # check very carefully which columns to sum etc in the loop below!
for (i in 1:nrow(table_simpl)) {
  table_simpl$sound[i] = levels(df1$sound)[i]
  temp = subset (df1, df1$sound==levels(df1$sound)[i])
  table_simpl[i,2:7] = apply (temp[, 6:11], 2, FUN=function(x) median(x, na.rm=T))  # mean produces outliers, esp. for Russian
  table_simpl[i,8:(length(calls)+length(ems)+8-1)] = apply (temp[, 12:(length(calls)+length(ems)+12-1)], 2, sum)
  table_simpl$topCall[i] = names (which.max(table_simpl[i, 8 : (length(calls)+8-1)]))
  table_simpl$topEm[i] = names (which.max(table_simpl[i, (length(calls)+8) : (length(calls)+8+length(ems)-1)]))
  table_simpl$topCallShare[i] = max(table_simpl[i, 8 : (length(calls)+8-1)]) / sum(table_simpl[i,8 : (length(calls)+8-1)])
  table_simpl$topEmShare[i] = max(table_simpl[i, (length(calls)+8) : (length(calls)+8+length(ems)-1)]) / sum(table_simpl[i, (length(calls)+8) : (length(calls)+8+length(ems)-1)])
}

## entropy
for (i in 1:nrow(table_simpl))  {
  a = table_simpl[i , 8 : (length(calls)+8-1)] + 1e-08  # to avoid zeroes 
  b = table_simpl[i , (length(calls)+8) : (length(calls)+8+length(ems)-1)] + 1e-08
  table_simpl$callEntropy[i] = -sum(log2(a/sum(a))*a/sum(a)) / log(length(calls))*log(2)*100
  table_simpl$emEntropy[i] = -sum(log2(b/sum(b))*b/sum(b)) / log(length(ems))*log(2)*100  
}

table(table_simpl$topCall)
# write.csv (table_simpl, 'table_simpl_sv_for-time.csv')
# write.csv (df1, 'df1_ru.csv')
# table(df1$order)/132
# length(unique(df1$id))
# length(table(df1$id)[table(df1$id)>100])
# write.csv (table(table_simpl$topCall, table_simpl$topEm), '~/Downloads/temp.csv')
