### Response times - linear regression in STAN
setwd (dir = '/home/allgoodguys/Documents/Studying/Lund_PhD/experiments/focal-terms/analysis')
require(brms)
require(shinystan)
library (reshape2)
library(lme4)

# data
df_sv = read.csv('df1_sv_for-time.csv') [, c('id','order','lang','sound','timeSound','timeEmotion','time','topCall','topEm')]
df_en = read.csv('df1_en_for-time.csv') [, c('id','order','lang','sound','timeSound','timeEmotion','time','topCall','topEm')]
df_ru = read.csv('df1_ru_for-time.csv') [, c('id','order','lang','sound','timeSound','timeEmotion','time','topCall','topEm')]
df_pooled = rbind (df_sv, df_en, df_ru)
summary(df_pooled$time/1000)
summary(df_pooled$time[df_pooled$lang!='ru']/1000)
nrow(df_pooled[df_pooled$time>60000,]) / nrow(df_pooled) # check that no times>60 s are left in the database

# how long did it take, on average, to choose the first sound OR emotion name?
timeFirst = apply (df_pooled[,c('timeSound','timeEmotion')], 1, function(x)min(x, na.rm=T)) 
summary(timeFirst/1000)

df_long = melt (df_pooled[,c('id','sound','order','timeSound','timeEmotion')])
colnames(df_long) = c('id','sound','order','namingTask','respTime')

hist(df_long$respTime)
df_long$respTime = log(df_long$respTime)
# range (df_long$respTime, na.rm=T)


# effect of condition and sound/emotion name on response time
summary(glmer(respTime~namingTask+order+ (1|sound) + (1|id), df_long))
modBRMS_cat = brm (respTime~namingTask+order + (1|sound) + (1|id), data = df_long, n.warmup=100, n.iter=1000, n.chains=4, n.cluster=4, family='gaussian', ranef=F) 
plot(modBRMS_cat)
summary(modBRMS_cat)
coda = posterior_samples(modBRMS_cat)
colnames(coda)
quantile (exp(coda[,'b_ordersoundFirst']), probs=c(.025,.5,.975)) 


# effect of starting with sound/emotion name on total time needed for both tasks
df_pooled$startWithSound = ifelse (df_pooled$timeSound<df_pooled$timeEmotion, T, F)
summary(glmer(time~startWithSound+order+ (1|sound) + (1|id), df_pooled))

