### Subjective certainty - ordinal logistic regression with stan 
setwd (dir = '/home/allgoodguys/Documents/Studying/Lund_PhD/experiments/focal-terms/analysis')
require(brms)
require(shinystan)

# data
df_sv = read.csv('df1_sv.csv') [, c('id','order','lang','sound','sureSound','sureEmotion','topCall','topEm', 'callDK', 'emDK')]
df_en = read.csv('df1_en.csv') [, c('id','order','lang','sound','sureSound','sureEmotion','topCall','topEm', 'callDK', 'emDK')]
df_ru = read.csv('df1_ru.csv') [, c('id','order','lang','sound','sureSound','sureEmotion','topCall','topEm', 'callDK', 'emDK')]
df = rbind (df_sv, df_en, df_ru)
df = na.omit(df)

# check whether the same results hold for all call types, emotions and lang-s
boxplot ( (sureSound-sureEmotion)~topCall, df)
aggregate ( (sureSound-sureEmotion)~topCall, df, mean)
boxplot ( (sureSound-sureEmotion)~topEm, df)
aggregate ( (sureSound-sureEmotion)~topEm, df, mean)
table (df_sv$sureSound)
table (df_sv$sureEmotion)
table (df_en$sureSound)
table (df_en$sureEmotion)
table (df_ru$sureSound)
table (df_ru$sureEmotion)

library(reshape2)
df = melt(df[,1:6])

# model
aggregate (value~variable, df, mean)
modBRMS_cat = brm (value~variable + (1|sound) + (1|id), data = df, n.warmup=100, n.iter=1000, n.chains=4, n.cluster=4, family='cumulative')
# stancode (modBRMS_cat)
# saveRDS (modBRMS_cat, 'sure_stan_mod.RDS')

plot(modBRMS_cat)
summary(modBRMS_cat)
launch_shiny(modBRMS_cat)

coda = posterior_samples(modBRMS_cat)[,1:3]
colnames(coda)

# manual prediction. See http://stats.stackexchange.com/questions/41006/predicting-ordered-logit-in-r 
logit1_s = coda[,1]
logit1_em = coda[,1] - coda[,3]
logit2_s = coda[,2] 
logit2_em = coda[,2] - coda[,3]
pLeq1_s  <- 1 / (1 + exp(-logit1_s))   # p(Y <= 1)
pLeq1_em  <- 1 / (1 + exp(-logit1_em))   # p(Y <= 1)
pLeq2_s  <- 1 / (1 + exp(-logit2_s))   # p(Y <= 2)
pLeq2_em  <- 1 / (1 + exp(-logit2_em))   # p(Y <= 2)
coda$dk_s = pLeq1_s
coda$dk_em = pLeq1_em
coda$unsure_s = pLeq2_s-pLeq1_s
coda$unsure_em = pLeq2_em-pLeq1_em
coda$sure_s = 1 - pLeq2_s
coda$sure_em = 1 - pLeq2_em
t (apply (coda[,4:9], 2, function(x)quantile(x, probs=c(.025,.5,.975))) )

df_plot = data.frame ( certainty = rep(c('DK','Unsure','Sure'), each=2), measure = rep(c('Sound','Emotion'),3) )
df_plot[,3:5] = t (apply (coda[,4:9], 2, function(x)quantile(x, probs=c(.025,.5,.975))) )
colnames(df_plot)[3:5] = c('lwr','fit','upr')
write.csv(df_plot, 'temp.csv'); df_plot=read.csv('temp.csv')[,-1]
df_plot$certainty = factor(df_plot$certainty, levels=c('DK','Unsure','Sure'))
df_plot$measure = factor(df_plot$measure, levels=c('Sound','Emotion'))
df_plot[,3:5] = df_plot[,3:5] * 100

library(ggplot2)
ggplot (df_plot, aes(x=certainty, y=fit, fill=measure)) +
  geom_bar(stat='identity', position=position_dodge()) + 
  geom_errorbar(aes(ymin=lwr, ymax=upr), position=position_dodge()) +
  xlab("") + ylab("%") + theme_bw () +
  scale_fill_manual(values=c('gray50','gray80'), name="", 
                    labels=c('Sound name', 'Emotion name')) +
  scale_x_discrete(labels=c("Don't know", "Unsure", "Sure")) +
  theme(legend.justification=c(0,1), legend.position=c(0,1),
        panel.grid.minor=element_blank(),
        panel.grid.major=element_blank())

ggsave("pix/sure.png", dpi=300, units='cm', width=9, height=6)

# sanity check
prop.table (table(df$value, df$variable), margin=2)

## how often participants answered 'dk' for sound but not emotion, and vice versa
df = rbind (df_sv, df_en, df_ru)
df = na.omit(df)
table(df$callDK, df$emDK)
nrow(df[df$callDK==1,]) / nrow(df) * 100
nrow(df[df$callDK==1 & df$emDK==0,]) / nrow(df)  * 100
nrow(df[df$emDK==1,]) / nrow(df)  * 100
nrow(df[df$callDK==0 & df$emDK==1,]) / nrow(df)  * 100

df$callNOemYES = ifelse(df$callDK==0 & df$emDK==1, 1, 0)
df$callYESemNOT = ifelse (df$callDK==1 & df$emDK==0, 1, 0)
df_mod = melt(df[,c(1:4, 11, 12)])
summary (glmer (value~variable + (1|id) + (1|sound), df_mod, 'binomial'))
mod = brm (value~variable + (1|sound) + (1|id), data = df_mod, n.warmup=100, n.iter=300, n.chains=4, n.cluster=4, family='bernoulli')
plot(mod)
a = summary(mod)
1/exp(a$fixed)

## correlation b/w sureSound and sureEmotion
temp_sv = read.csv('table_simpl_sv.csv')[,c('sound','sureSound','sureEmotion')]
temp_en = read.csv('table_simpl_en.csv')[,c('sound','sureSound','sureEmotion')]
temp_ru = read.csv('table_simpl_ru.csv')[,c('sound','sureSound','sureEmotion')]
temp_pooled = array (c(as.matrix(temp_sv),as.matrix(temp_en),as.matrix(temp_ru)), dim=c(132,3,3))
table_simpl = apply (temp_pooled[,2:3,], c(1,2), function(x) mean(as.numeric(x)))
table_simpl = data.frame ('sound'=temp_sv[,1], 'sureSound'=table_simpl[,1], 'sureEmotion'=table_simpl[,2])

summary (lm(sureSound~sureEmotion,table_simpl))
cor(table_simpl$sureSound, table_simpl$sureEmotion)
# to compare: individual responses are less strongly correlated: cor(log(df1$timeSound), log(df1$timeEmotion))

mod = brm (sureSound~sureEmotion, data = table_simpl, n.warmup=100, n.iter=5000, n.chains=4, n.cluster=4, family='gaussian') 
plot (mod)
summary(mod)
coda = posterior_samples(mod)
colnames(coda)
coda$cor = coda[,2] * sd(table_simpl$sureEmotion) / sd(table_simpl$sureSound)
quantile(coda$cor, probs=c(.025,.5,.975)) # Pearson's r with 95% CI


