setwd (dir = '/home/allgoodguys/Documents/Studying/Lund_PhD/experiments/focal-terms/analysis')
library(ggplot2)
library (nnet)
library(randomForest)

dfClust = read.csv('weighted-ac-table.csv')[,-1]
allColors=c('#d376dc','#903099','#dd345f','#42268d','#5c36ca','#c53fe1','#d43e88','#dd3eb7','#6de56f','#a85ae3','#b2d63f','#dab433','#d86f27','#52a437','#d03f30','#6a6de0')
key = data.frame (allCalls = c("cry","DK","gasp","giggle","groan","growl","grunt","howl","laugh","moan","roar","scream","shriek","sigh","sob","yell"), allShapes = c(0:5,8:13,15:18), allColors=c('#d376dc','#903099','#dd345f','#42268d','#5c36ca','#c53fe1','#d43e88','#dd3eb7','#6de56f','#a85ae3','#b2d63f','#dab433','#d86f27','#52a437','#d03f30','#6a6de0')) # allColors = rainbow(16, start=1/16, end=0))
allvars = c('voiced','ampl_median','entropy_sd','pitch_manually','pitch_sd','quartile25_median','quartile25_sd','syllableLength_median','syllableLength_sd','noBursts','interBurst_median','interBurst_sd')


####

method = c('RF','multinomial')[1]
lang = c('en','sv','ru')[2]

if (lang=='sv'){
  df1 = read.csv ('df1_sv.csv')[,-1]
  table_simpl = read.csv ('table_simpl_sv.csv')[,-1]
  table_simpl$topCall = as.character(table_simpl$topCall)
  table_simpl$topCall[table_simpl$topCall=='callDK'] = 'DK'
  table_simpl$topCall = as.factor(table_simpl$topCall)
  calls = levels(table_simpl$topCall)
  calls_en = c ('DK', 'gasp', 'giggle', 'cry', 'yell', 'laugh', 'scream', 'moan', 'sigh', 'howl', 'roar')
} else if (lang=='en') {
  df1 = read.csv ('df1_en.csv')[,-1]
  table_simpl = read.csv ('table_simpl_en.csv')[,-1]
  table_simpl$topCall = as.character(table_simpl$topCall)
  table_simpl$topCall[table_simpl$topCall=='callDK'] = 'DK'
  table_simpl$topCall = as.factor(table_simpl$topCall)
  calls = levels(table_simpl$topCall)
  calls_en = calls
} else if (lang=='ru'){
  df1 = read.csv ('df1_ru.csv')[,-1]
  table_simpl = read.csv ('table_simpl_ru.csv')[,-1]
  table_simpl$topCall = as.character(table_simpl$topCall)
  table_simpl$topCall[table_simpl$topCall=='callDK'] = 'DK'
  table_simpl$topCall = as.factor(table_simpl$topCall)
  calls = levels(table_simpl$topCall)
  calls_en = c('DK', 'sigh', 'shriek', 'howl', 'yell', 'scream', 'grunt', 'cry', 'roar', 'laugh', 'moan', 'giggle') 
}

langVar = paste0('topCall_',lang)
rownames(dfClust) = paste0(1:nrow(dfClust),'_',table_simpl$topCall)

## prcomp
prc = prcomp(dfClust, scale=F)
plot(prc)
summary(prc)$importance[,1:10]
biplot(prc, col=c('gray80','red'))
prc$rotation[,1:2] # var loadings write.csv (round(prc$rotation[,1:2],2), '~/Downloads/temp.csv')

table_simpl$pc1 = -prc$x[,1]; table_simpl$pc2 = -prc$x[,2]


## Classification model
if (method=='multinomial'){
  mod <- multinom(topCall~pc1*pc2, table_simpl) # summary(mod)
} else if (method=='RF'){
  mod_rf = randomForest (topCall~pc1+pc2, table_simpl)
}

# set the range of predictors
predRange = matrix(nrow=100,ncol=2)
colnames (predRange) = c('pc1','pc2')ggsave (filename, height=5, width=6, scale=1.2)
predRange[,1] = seq(range(table_simpl$pc1)[1], range(table_simpl$pc1)[2], length.out=100)
predRange[,2] = seq(range(table_simpl$pc2)[1], range(table_simpl$pc2)[2], length.out=100)

## plotting predicted class rather than probabilities of belonging to each class
output = expand.grid (list(predRange[,1], predRange[,2]))
colnames(output) = c('pc1','pc2')
if (method=='multinomial'){
  output$topCall = as.character ( predict (mod, output[,1:2], "class") )
} else if (method=='RF'){
  output$topCall = as.character ( predict (mod_rf, output[,1:2]) )
}



# Get centroids of predicted regions for each call
dfPlot = data.frame ('call'=calls, 'call_en'=calls_en, 'pc1'=NA, 'pc2'=NA)
output$topCall = factor(output$topCall, levels=unique(dfPlot$call[order(dfPlot$call_en)]))
for (l in 1:length(calls)) {  # for each call type or emotion
  mySubset = output[output$topCall==calls[l],c('pc1','pc2')]
  dfPlot[l,3:4] = apply(mySubset, 2, median) # for discontinuous areas, try hist(mySubset[,1]); hist(mySubset[,2])
}

## plotting
dfPlot$call = factor(dfPlot$call, levels=unique(dfPlot$call[order(dfPlot$call_en)]))
table_simpl$topCall = factor(table_simpl$topCall, levels=unique(dfPlot$call[order(dfPlot$call_en)]))
myCols = as.character(key$allColors[match(levels(dfPlot$call_en), key$allCalls)])

# make translations
if (lang=='en'){
  dfPlot$call_noDK = rep('', nrow(dfPlot))
  mylabels = levels(dfPlot$call)
} else {
  dfPlot$call_noDK = paste0('(',as.character(dfPlot$call),')')
  dfPlot$call_noDK[dfPlot$call_noDK=='(DK)'] = ''
  mylabels = paste(levels(dfPlot$call_en),dfPlot$call_noDK[order(dfPlot$call_en)])
}

# manual correction for discontinous areas
if (method=='multinomial'){
  if (lang=='ru'){
    dfPlot[dfPlot$call=='вопль', c('pc1','pc2')] = c(0.3, -0.8)
    dfPlot = rbind (dfPlot, c('вопль','yell',-4.5,7,'(вопль)'))
    dfPlot$pc1=as.numeric(dfPlot$pc1); dfPlot$pc2=as.numeric(dfPlot$pc2)
  } else if (lang=='sv'){
    dfPlot[dfPlot$call=='rop', c('pc1','pc2')] = c(-5, 8)
    dfPlot = rbind (dfPlot, c('rop','yell',4,-2.5,'(rop)'))
    dfPlot$pc1=as.numeric(dfPlot$pc1); dfPlot$pc2=as.numeric(dfPlot$pc2)
  } else if (lang=='en'){
    dfPlot$call_noDK=''
    dfPlot = rbind (dfPlot, c('yell','yell',5,-2.5,''))
    dfPlot$pc1=as.numeric(dfPlot$pc1); dfPlot$pc2=as.numeric(dfPlot$pc2)
  }
}

ggplot(output, aes(x=pc1, y=pc2, z=topCall, fill=topCall)) + 
  geom_tile (alpha=.75) +
  scale_fill_manual (values=myCols, drop=F) +
  geom_text (data=table_simpl, aes(x=pc1, y=pc2, label=topCall), col='gray25', size=4, alpha=.5) +
  scale_color_manual (values=myCols, drop=F) +
  geom_text (data=dfPlot, aes(x=pc1, y=pc2, label=call_en), col='black', size=7, inherit.aes=F) + 
  geom_text (data=dfPlot, aes(x=pc1, y=pc2-.5, label=call_noDK), col='black', size=5, inherit.aes=F) + 
  coord_fixed() + # force square shape of graph area
  xlab("PC1") + ylab("PC2") +
  theme(legend.position="none",
        panel.background = element_rect(color='gray50', fill='white'),
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank())

        # axis.ticks = element_blank(),
        # axis.text = element_blank(),
        # panel.border = element_blank(),
        # panel.background = element_rect(color='white')) 

filename = paste0('/home/allgoodguys/Documents/Studying/Lund_PhD/epistles/03_article_labels/pix/PC_acoustic_contour_', method, '_', lang, '.png')
# 
