# Cluster analysis (doesn't really work, ie the clusters do not correspond to either call types or emotion, plus it's very hard to know which acoustic variables to pick and what to do with NA values). Try to do the same based on dist matrix calculated from co-occurrence of labels, compare with acoustics (should be vastly clearer clusters). Same for emotion?

setwd (dir = '/home/allgoodguys/Documents/Studying/Lund_PhD/experiments/focal-terms/analysis')
library (mclust)
library (apcluster)

acoustics = read.csv ('all_july2016.csv')
labels = read.csv ('table_simpl_pooled.csv')
labels$sound = substr(labels$sound, 1, nchar(as.character(labels$sound))-4)
varsToLog = c('OLD.manual','pitch_manually','pitch_median','pitch_sd','quartile25_median','quartile25_sd','quartile50_median','quartile50_sd','quartile75_median','quartile75_sd','dom_median','dom_sd','meanFreq_median','meanFreq_sd','peakFreq_median','peakFreq_sd','peakFreq_cut_median','peakFreq_cut_sd','pitchAutocor_median','pitchAutocor_sd','pitchCepstrum_median','pitchCepstrum_sd','pitchSpec_median','pitchSpec_sd')
acoustics[,varsToLog] = log2(acoustics[,varsToLog])



### Optimization: iteratively choose the best combination of acoustic predictors to obtain clusters similar to human labeling (in one of three tested languages)
# allvars = c('duration','voiced','ampl_median','ampl_sd','ampl_voiced_median','ampl_voiced_sd','entropy_median','entropy_sd','HNR_median','HNR_sd','dom_median','dom_sd','meanFreq_median','meanFreq_sd','peakFreq_median','peakFreq_sd','peakFreq_cut_median','peakFreq_cut_sd','pitch_median','pitch_sd','pitchAutocor_median','pitchAutocor_sd','pitchCepstrum_median','pitchCepstrum_sd','pitchSpec_median','pitchSpec_sd','quartile25_median','quartile25_sd','quartile50_median','quartile50_sd','quartile75_median','quartile75_sd','specSlope_median','specSlope_sd','harmonics_median','harmonics_sd','pitch_manually','OLD.manual','noSyllables','syllableLength_mean','syllableLength_median','syllableLength_sd','noBursts','interBurst_mean','interBurst_median','interBurst_sd') # paste(colnames(acoustics)[14:59], collapse="','")
allvars = c('duration','voiced','ampl_median','ampl_sd','entropy_median','entropy_sd','HNR_median','HNR_sd','dom_median','dom_sd','meanFreq_median','meanFreq_sd','peakFreq_median','peakFreq_sd','pitch_manually','pitch_sd','quartile25_median','quartile25_sd','quartile50_median','quartile50_sd','quartile75_median','quartile75_sd','specSlope_median','specSlope_sd','harmonics_median','harmonics_sd','noSyllables','syllableLength_median','syllableLength_sd','noBursts','interBurst_median','interBurst_sd')
lang = c('en','sv','ru')[2]
nIter = 50000


langVar = paste0('topCall_',lang)
labels = read.csv ('table_simpl_pooled.csv')
labels$sound = substr(labels$sound, 1, nchar(as.character(labels$sound))-4)
labels_clusters = as.numeric(labels[,langVar])
a = match (labels$sound, acoustics$name132)
df = cbind (acoustics[a,allvars], labels)
df[,1:length(allvars)] = scale(df[,1:length(allvars)]) # scale all vars
df[is.na(df)] = 0


## optimization by brute force - trying many-many different combinations of vars
output = data.frame('nPred'=rep(NA,nIter), 'nCl'=NA, 'purity'=NA, 'adjustedRandIndex'=NA, 'myvars'=NA)
for (i in 1:nIter){
  myvars = sort (sample (allvars, size=floor(runif(n=1, min=1, max=length(allvars))))) # grab a random number of acoustic vars out of the list to be used for clustering. Sort so that the sequences can be easily compared to remove duplicates etc
  # myvars = c('pitch_median','interBurst_median','syllableLength_median','syllableLength_sd','pitch_sd','HNR_median','HNR_sd','entropy_median', 'interBurst_median','ampl_median','specSlope_sd')
  nPred = length(myvars)
  temp = df[,myvars, drop=F]
  rownames(temp) = paste0(1:nrow(temp),'_',labels[,langVar])
  
  # affinity propagation clustering
  sim <- negDistMat(temp, r=2) 
  apres <- apcluster(sim, details=TRUE)
  # plot(apres)
  aggres <- aggExCluster(sim, apres)
  myLevels = unique(labels[,langVar])
  nLevelsUsed = min(length(myLevels),aggres@maxNoClusters)
  tr = cutree(aggres, k=nLevelsUsed) # divide into as many clusters as there are linguistic labels in this language (ie 12 for sv and ru, 16 for en) or the max number of clusters detected by the algorithm, whichever is smaller
  
  # calculate cluster purity my way
  purity = numeric()
  for (cl in 1:nLevelsUsed){
    myCluster = tr@clusters[[cl]]
    lbs = as.character (labels[myCluster,langVar])
    purity = c(purity, max(table(lbs)) / length(lbs)) # how many sounds in the cluster have the same name?
  }
  output$purity[i] = weighted.mean(purity, w=sapply(tr@clusters, length)) # take a weighted mean of cluster purities (weights are cluster sizes)
  
  # calculate adjusted Rand Index to compare this clustering with human labeling (in one chosen language)
  c = tr@clusters 
  cluster_out = 1:132
  for (j in 1:length(c)){
    cluster_out[c[[j]]] = j # convert a list of clusters to a simple list of cluster numbers to which each of 132 sounds belong...
  }
  output$adjustedRandIndex[i] = adjustedRandIndex(cluster_out, labels_clusters) # ...and compare this 132-long vector to the human labeling
  
  output$myvars[i] = paste (myvars, collapse=',')
  output$nPred[i] = length(myvars)
  output$nCl[i] = aggres@maxNoClusters
  
  if (i%%100==0) {print (paste('Done with iteration', i))}
}
output = output[order(output$adjustedRandIndex, decreasing = T),]
output = output[!duplicated(output$myvars),] # remove duplicates, if any
write.csv(output, paste0('~/Downloads/temp_',lang,'.csv'))

# analyze top acoustic vars
v = strsplit (output$myvars[1:100],',')
bestVars = sort (table(unlist(v)), decreasing = T)
write.csv (as.matrix(bestVars), paste0('~/Downloads/temp_',lang,'_topAcVars.csv'))

plot (aggregate(purity~nPred,output,mean))
plot (aggregate(purity/nCl~nPred,output,mean))
plot (aggregate(adjustedRandIndex~nPred,output,mean))



# 
# ********************************************
# FINAL ANALYSIS
# ********************************************
#

### affinity propagation and exemplar-based agglomerative clustering
library (apcluster)
allvars = c('voiced','ampl_median','entropy_sd','pitch_manually','pitch_sd','quartile25_median','quartile25_sd','syllableLength_median','syllableLength_sd','noBursts','interBurst_median','interBurst_sd')
# allvars = c('pitch_manually','HNR_median','voiced','quartile75_median','meanFreq_median','duration','dom_median','entropy_sd','specSlope_sd','quartile25_median','noBursts','entropy_median','syllableLength_sd','harmonics_median','peakFreq_median','quartile50_sd','specSlope_median','ampl_median','syllableLength_median')
# allvars = c('duration','voiced','ampl_median','ampl_sd','entropy_median','entropy_sd','HNR_median','HNR_sd','dom_median','dom_sd','meanFreq_median','meanFreq_sd','peakFreq_median','peakFreq_sd','pitch_manually','pitch_sd','quartile25_median','quartile25_sd','quartile50_median','quartile50_sd','quartile75_median','quartile75_sd','specSlope_median','specSlope_sd','harmonics_median','harmonics_sd','noSyllables','syllableLength_median','syllableLength_sd','noBursts','interBurst_median','interBurst_sd')
# allvars = c('interBurst_sd','noBursts','dom_median','quartile25_median','voiced')
# allvars = c('dom_median','noBursts')

lang = c('en','sv','ru')[3]


langVar = paste0('topCall_',lang)
labels = read.csv ('table_simpl_pooled.csv')
labels$sound = substr(labels$sound, 1, nchar(as.character(labels$sound))-4)
labels_clusters = as.numeric(labels[,langVar])
a = match (labels$sound, acoustics$name132)
df = cbind (acoustics[a,allvars], labels) # or: df = read.csv('weighted-ac-table.csv')[,-1]; colnames(df)
df[,1:length(allvars)] = scale(df[,1:length(allvars)]) # scale all vars
df[is.na(df)] = 0

dfClust = df[,allvars]
# or: dfClust = read.csv('weighted-ac-table.csv')[,-1]
rownames(dfClust) = paste0(1:nrow(dfClust),'_',labels[,langVar])
dm = dist(dfClust) # plot (prcomp(dist(dfClust)))
mds = cmdscale(dm) # plot(mds); text(mds, labels=rownames(dfClust), cex=.75)
# cor(cbind(mds, dfClust))

## compute similarity matrix (negative squared Euclidean)
sim <- negDistMat(dfClust, r=2) # sim <- as.matrix(1/dist(dfClust))
## run affinity propagation
apres <- apcluster(sim, details=TRUE)
## show details of clustering results
show(apres)
## plot information about clustering run
plot(apres)
## plot clustering result
plot(apres, mds)
text(mds, labels=rownames(dfClust), cex=.75)

## employ agglomerative clustering to join clusters
aggres <- aggExCluster(sim, apres)
## plot dendrogram
plot(aggres)
## plot clustering result for k clusters
nClust = 6 # length(unique(labels[,langVar]))
plot(aggres, mds, k=nClust)
text(mds, labels=rownames(dfClust), cex=.75)
## print a list of files in each cluster
show(cutree(aggres, k=nClust))


library(randomForest)
df_rf = df
df_rf$label = labels[,langVar]
mod = randomForest (as.formula(paste('label', '~',paste0(allvars,collapse='+'))), df_rf) # strata=label, sampsize=rep(min(table(df_rf$label)),length(unique(df_rf$label))))
mod # mod$confusion[,ncol(mod$confusion)]
varImpPlot(mod)   # with 12 vars, about 62% correct for sv, 54% for ru, 40% for en). To vars: c('pitch_manually','pitch_sd','quartile25_median','interBurst_median','voiced','ampl_median') 
# chance level: 1/16 for en, 1/11 for sv, 1/13 for ru


## compare en/sv/ru naming as silhouette of clusters
adjustedRandIndex(as.numeric(labels$topCall_en), as.numeric(labels$topCall_sv))
adjustedRandIndex(as.numeric(labels$topCall_en), as.numeric(labels$topCall_ru))
adjustedRandIndex(as.numeric(labels$topCall_sv), as.numeric(labels$topCall_ru))

adjustedRandIndex(as.numeric(labels$topEm_en), as.numeric(labels$topEm_sv))
adjustedRandIndex(as.numeric(labels$topEm_en), as.numeric(labels$topEm_ru))
adjustedRandIndex(as.numeric(labels$topEm_sv), as.numeric(labels$topEm_ru))

# calculate silhouette for clustering of averaged distance matrices based on naming in three languages and the actual naming in each language
distMatS_sv = readRDS ('distMatS_sv.RDS')
distMatS_en = readRDS ('distMatS_en.RDS')
distMatS_ru = readRDS ('distMatS_ru.RDS')

sim = as.matrix( -(distMatS_en+distMatS_sv+distMatS_ru)/3 )
rownames(sim)=colnames(sim)=labels$sound
apres <- apcluster(sim, details=TRUE)
aggres <- aggExCluster(sim, apres)

lang = c('en','sv','ru')[3]
langVar = paste0('topCall_',lang)
labels_clusters = as.numeric(labels[,langVar])
nClust = min(aggres@maxNoClusters,length(unique(labels_clusters)))
tr = cutree(aggres, k=nClust)
c = tr@clusters 
cluster_out = 1:132
for (j in 1:length(c)){
  cluster_out[c[[j]]] = j # convert a list of clusters to a simple list of cluster numbers to which each of 132 sounds belong...
}  
adjustedRandIndex(cluster_out, labels_clusters) # en 0.46, sv 0.49, ru 0.48


## To do: working with the chosen 12 vars with weights, cluster/MDS 590 sounds (just to try)