
library (apcluster)
library (cluster)
library (mclust)
setwd (dir = '/home/allgoodguys/Documents/Studying/Lund_PhD/experiments/focal-terms/analysis')

lang = c('en','sv','ru')[1]

if (lang=='sv'){
  df = read.csv('table_simpl_sv.csv')
  calls = c('flämtning','fniss','gnäll','gråt','rop','skratt','skrik','snyftning','stön','suck','tjut','vrål','callDK')
  ems = c('ansträngning','frustration','glädje','hysteri','ilska','irritation','jubel','lättnad','munterhet','njutning','panik','rädsla','smärta','sorg','äckel','överraskning','emDK')
} else if (lang=='en') {
  df = read.csv('table_simpl_en.csv')
  calls = c('chuckle','cry','gasp','giggle','groan','growl','grunt','laugh','moan','roar','scream','shriek','sigh','snort','sob','yell','callDK')
  ems = c('amusement','anger','annoyance','disgust','effort','excitement','fear','fright','frustration','happiness','hysteria','pain','pleasure','relief','sadness','surprise','emDK')
} else if (lang=='ru'){
  df = read.csv('table_simpl_ru.csv')
  calls = c('вздох','визг','вой','вопль','крик','кряхтенье','нытье','плач','рев','смех','стон','хихиканье','callDK')
  ems = c('боль','веселье','восторг','грусть','досада','злость','испуг','истерика','облегчение','отвращение','радость','раздражение','страх','удивление','удовольствие','усилие','emDK')
}

dfClust = df[,calls]
rownames(dfClust) = paste0(1:nrow(df),'_',df$topCall)

## compute similarity matrix (negative squared Euclidean)
sim <- negDistMat(dfClust, r=2) # sim =  # readRDS('distMatS_en.RDS')
mds = cmdscale(sim, k=2)


## optimize q based on silhouette
langVar = paste0('topCall_',lang)
labels_clusters = as.numeric(df$topCall)
qs = seq(0.01, 0.99, length.out=99)
out = data.frame ('q'=qs, 'silhouette'=NA, 'adjustedRandIndex'=NA, 'nClust'=NA)
for (i in 1:length(qs)){
  apres <- apcluster(sim, details=TRUE, q=qs[i])
  out$nClust[i] = length(apres@exemplars)
  c = apres@clusters # or: cluster_out = cutree(aggres, k=10)
  cluster_out = 1:132
  for (j in 1:length(c)){
    cluster_out[c[[j]]] = j
  }
  names(cluster_out) = rownames(dfClust)
  out$adjustedRandIndex[i] = adjustedRandIndex(cluster_out, labels_clusters)
  sil = silhouette (x=cluster_out, dist=-sim) # dist=dist(dfClust)
  s = summary(sil)
  out$silhouette[i] = weighted.mean (s$clus.avg.widths, w=s$clus.sizes)
}
plot (out$q, out$silhouette, type='l') # optimal for all three langs - about .3
plot (out$q, out$adjustedRandIndex, type='l') # optimal for en: .75 or .82; for sv: .5; for ru: .35
plot (out$q, out$silhouette*out$adjustedRandIndex, type='l')
out[which.max(out$silhouette*out$adjustedRandIndex),] # optimal for en .68 (13 clusters); for sv .5 (9 clusters); for ru .35 (7 clusters)

## optimize number of clusters based on silhouette
out = data.frame ('nClust'=2:16, 'silhouette'=NA)
apres <- apcluster(sim, details=TRUE, q=.3)
aggres <- aggExCluster(sim, apres)

for (i in 2:16){
  c = cutree(aggres, k=i)@clusters
  cluster_out = 1:132
  for (i in 1:length(c)){
    cluster_out[c[[i]]] = i
  }
  names(cluster_out) = rownames(dfClust)
  sil = silhouette (x=cluster_out, dist=-sim) # dist=dist(dfClust)
  s = summary(sil)
  out$silhouette[i] = weighted.mean (s$clus.avg.widths, w=s$clus.sizes)
}
plot (out, type='l') # optimal: 8 for ru, 5 or 9 for en, 5 for sv




# **************************************
### Final analysis with optimized values
# **************************************

## run affinity propagation
apres <- apcluster(sim, details=TRUE, q=.5) # .5 for en, .3 for sv, .45 for ru
## show details of clustering results
show(apres)
## plot information about clustering run
plot(apres)
## plot clustering result
plot(apres, mds)
text(mds, labels=df$topCall, cex=.75)

## employ agglomerative clustering to join clusters
aggres <- aggExCluster(sim, apres)
## plot dendrogram
plot(aggres)

## plot clustering result for k clusters
plot(aggres, mds, k=7)
text(mds, labels=df$topCall, cex=.75)
## print a list of files in each cluster
show(cutree(aggres, k=10)) # for sv, 6; for ru, 7; for en, 

show(cutree(aggres, k=7))

# if needed, re-label terminal nodes:
nClust = 14
show(cutree(aggres, k=nClust))

dendrogram = plot(aggres)
newLabels = names(aggres@exemplars[[nClust]])[aggres@order]
# for sve: newLabels = c('Scream\n(skrik)', 'Roar\n(vrål)', 'Sigh (flämtning\n/suck/stön)', 'DK/vrål/stön', 'Moan\n(stön)', 'Laugh\n(skratt)', 'Cry\n(gråt)')
# for en: newLabels = c('Shriek','Roar','Scream','Sigh','Moan','Grunt','Groan','DK','Gasp','Laugh','Cry')
# for ru: newLabels = c('Moan\n(стон)', 'Sigh\n(вздох)', 'Scream\n(крик)', 'Shriek\n(визг)', 'Grunt\n(кряхтенье)', 'Roar\n(рёв)', 'Yell\n(вопль)', 'Laugh\n(смех)', 'Cry\n(плач)')

local({
  newLab <<- function(n) {
    if(is.leaf(n)) {
      a <- attributes(n)
      i <<- i+1
      attr(n, "label") <- newLabels[i]
    }
    n
  }
  i <- 0
})

nhc <- dendrapply(dendrogram, newLab)
# labels(nhc)
# par('mar')
# par (mar=c(6,4,4,0))
# plot(nhc)

# par (mar=c(0,0,0,6))
plot(nhc, type='triangle', horiz=T, axes=F)



## get silhouette as a measure of clustering quality
c = apres@clusters # or: cluster_out = cutree(aggres, k=10)
cluster_out = 1:132
for (i in 1:length(c)){
  cluster_out[c[[i]]] = i
}
names(cluster_out) = rownames(dfClust)
sil = silhouette (x=cluster_out, dist=-sim) # dist=dist(dfClust)
s = summary(sil)
plot(sil)
weighted.mean (s$clus.avg.widths, w=s$clus.sizes) # average silhouette for this clustering solution

