require(rgl)
table_simpl = read.csv ('table_simpl_en.csv')[,-1]
table_simpl = read.csv ('table_simpl_sv.csv')[,-1]
table_simpl = read.csv ('table_simpl_ru.csv')[,-1]

## prcomp based on naming sounds
distMatS = dist (subset (table_simpl, select=calls[1:(length(calls))])) # -1 to remove dk
plot(prcomp(distMatS))
summary(prcomp(distMatS))$importance[,1:10]
# saveRDS (distMatS, 'distMatS_en.RDS')

# scores = summary(prcomp(distMatS))$x[,1:3]
scores = cmdscale(distMatS, 3)
plot3d (-scores[,1],-scores[,2],-scores[,3], col=as.numeric(as.factor(table_simpl$topCall))) 
text3d(-scores[,1],-scores[,2],-scores[,3], texts=table_simpl$topCall, cex=1, col=as.numeric(as.factor(table_simpl$topCall)))
# movie3d( spin3d(), duration=12, dir = '~/Downloads/' )  # make a movie. 12 gives a full 360° circle. 


## prcomp based on naming emotions
distMatEm = dist (subset (table_simpl, select=ems[1:(length(ems)-1)]))
# saveRDS (distMatEm, 'distMatEm_en.RDS')
plot(prcomp(as.dist(distMatEm)))
summary(prcomp(distMatEm))$importance[,1:10]



### Clustering quality - - - RUN EXTRACTION SCRIPT FOR EACH LANGUAGE!
clustering_var = c('calls','ems') [2]  # choose what clusters to examine
remove_dk = F   # preserve or remove callDK or emDK?

if (remove_dk) { # might want to exclude these sounds completely?
  mylevels = head(unique(get(clustering_var)),-1)
} else {
  mylevels = unique(get(clustering_var))
}

# get global centroid and median distance
if (clustering_var=='calls') {
  global_centroid = apply(table_simpl[,calls], 2, mean) # center of gravity of the entire cloud
  global_median_dist = median ( apply (table_simpl[,calls],1,function(x)dist(rbind(x,global_centroid))) ) # median distance from each sound to this global centroid
} else if (clustering_var=='ems') {
  global_centroid = apply(table_simpl[,ems], 2, mean)
  global_median_dist = median ( apply (table_simpl[,ems],1,function(x)dist(rbind(x,global_centroid))) )
}

# Get median inter-cluster distances and cluster purity
output = data.frame ('level'=mylevels)
for (l in 1:length(mylevels)) {  # for each call type or emotion
  if (clustering_var=='calls') {
    mySubset = table_simpl[table_simpl$topCall==mylevels[l],calls]
    weights = table_simpl[table_simpl$topCall==mylevels[l],'sureSound'] / table_simpl[table_simpl$topCall==mylevels[l],'callEntropy']; weights = weights/sum(weights)
    cluster_centroid = apply(mySubset, 2, function(x)sum(x*weights)) # NB: this is in 12 or 16 dimensions, not 3!
    distances_allS = apply (table_simpl[,calls],1,function(x)dist(rbind(x,cluster_centroid))) # distances from each of 132 sounds to the centroid of this cluster
  } else if (clustering_var=='ems') {
    mySubset = table_simpl[table_simpl$topEm==mylevels[l],ems]
    weights = table_simpl[table_simpl$topEm==mylevels[l],'sureSound'] / table_simpl[table_simpl$topEm==mylevels[l],'callEntropy']; weights = weights/sum(weights)
    cluster_centroid = apply(mySubset, 2, function(x)sum(x*weights))
    distances_allS = apply (table_simpl[,ems],1,function(x)dist(rbind(x,cluster_centroid)))
  }

  distances_correctS = apply (mySubset,1,function(x)dist(rbind(x,cluster_centroid))) # distances of each sound within this cluster to the centroid of this cluster
  noCorrectS = nrow(mySubset)
  noIncorrectS = nrow( table_simpl[distances_allS <= max(distances_correctS),] ) - noCorrectS # number of sounds from other categories that lie within the sphere that includes all sounds in the cluster (max distance makes sure all sounds in the cluster are included - it's the smallest sphere that contains the cluster entirely; we see what else we find in this sphere)
  
  output$count[l] = noCorrectS
  output$distance[l] = median(distances_correctS) / global_median_dist*100 # median distance between each sound in the cluster and its center, normalized by the global median distance
  output$purity[l] = noCorrectS/(noCorrectS+noIncorrectS)*100 # proportion of "correct" sounds in the cluster 
}

# Get entropy
for (l in 1:length(mylevels)) {  
  if (clustering_var=='calls') {
    mySubset = table_simpl[table_simpl$topCall==mylevels[l],]
    entropy = median (mySubset$callEntropy, na.rm=T)
  } else if (clustering_var=='ems') {
    mySubset = table_simpl[table_simpl$topEm==mylevels[l],]
    entropy = median (mySubset$emEntropy, na.rm=T)
  }
  output$entropy[l] = round (entropy, 2)
}

# Set distance and purity at NA for clusters with <2 sounds
output$distance = ifelse (output$count<2, NA, output$distance)
output$purity = ifelse (output$count<2, NA, output$purity)

# Get totals - weighted means
output = rbind (output, data.frame (level='TOTAL', 
         count=sum(output$count), 
         distance=sum(output$distance*output$count/sum(output$count), na.rm=T),# weighted mean
         purity=sum(output$purity*output$count/sum(output$count), na.rm=T),
         entropy=sum(output$entropy*output$count/sum(output$count), na.rm=T) ))
output$distance=round(output$distance,0)
output$purity=round(output$purity,0)
output$entropy=round(output$entropy,0)
output
# write.csv (output, '~/Downloads/temp.csv')

