setwd (dir = '/home/allgoodguys/Documents/Studying/Lund_PhD/experiments/focal-terms/analysis')
library(rgl)

distMatS_sv = readRDS ('distMatS_sv.RDS')
distMatS_en = readRDS ('distMatS_en.RDS')
distMatS_ru = readRDS ('distMatS_ru.RDS')

table_simpl_sv = read.csv ('table_simpl_sv.csv')[,-1]
table_simpl_en = read.csv ('table_simpl_en.csv')[,-1]
table_simpl_ru = read.csv ('table_simpl_ru.csv')[,-1]


## plotting with centroids 
lang = c('en','sv','ru')[2]

key = data.frame (allCalls = c("cry","DK","gasp","giggle","groan","growl","grunt","howl","laugh","moan","roar","scream","shriek","sigh","sob","yell"), allShapes = c(0:5,8:13,15:18), allColors=c('#d376dc','#903099','#dd345f','#42268d','#5c36ca','#c53fe1','#d43e88','#dd3eb7','#6de56f','#a85ae3','#b2d63f','#dab433','#d86f27','#52a437','#d03f30','#6a6de0'))
if (lang=='sv') {
  scores = cmdscale(distMatS_sv, 3)
  dfTemp = table_simpl_sv
  mycalls = c ('DK', 'gasp', 'giggle', 'cry', 'yell', 'laugh', 'scream', 'moan', 'sigh', 'howl', 'roar')
} else if (lang=='en') {
  scores = cmdscale(distMatS_en, 3)
  dfTemp = table_simpl_en
  mycalls = levels(dfTemp$topCall); mycalls[1] = 'DK'
  } else if (lang=='ru') {
  scores = cmdscale(distMatS_ru, 3)
  dfTemp = table_simpl_ru
  mycalls = c('DK', 'sigh', 'shriek', 'howl', 'yell', 'scream', 'grunt', 'cry', 'roar', 'laugh', 'moan', 'giggle') 
  }

dfTemp$mds1 = scores[,1]
dfTemp$mds2 = scores[,2]
dfTemp$mds3 = scores[,3]
dfTemp$topCall = as.character(dfTemp$topCall)
dfTemp$topCall = ifelse (dfTemp$topCall=='callDK','',dfTemp$topCall)
dfTemp$topCall = as.factor(dfTemp$topCall)

centroids = matrix(ncol=3, nrow=length(levels(dfTemp$topCall)))
for (l in 1:length(levels(dfTemp$topCall))) {
  a = dfTemp[dfTemp$topCall==levels(dfTemp$topCall)[l],]
  weights = a$sureSound*a$topCallShare; weights = weights/sum(weights)
  # centroids[l,] = apply (a[,c('mds1','mds2','mds3')], 2, mean) # unadjusted center of gravity
  centroids[l,] = apply (a[,c('mds1','mds2','mds3')], 2, function(x)sum(x*weights))
} # mean weighted by sureSound and /callEntropy - shows centroid taking certainty into account
colnames(centroids) = c('mds1','mds2','mds3')
centroids = as.data.frame(centroids)
centroids$topCall = levels(dfTemp$topCall)

# optional: rotate to standardize view
rot = c(1,2) # specify which axes to invert
for (r in rot){
  scores[,r] = -scores[,r] 
  centroids[,r] = -centroids[,r] 
}

## no box, axes
# zoom<-par3d()$zoom  # suggestion: zoom=0.52
# userMatrix<-par3d()$userMatrix # suggestion: userMatrix = matrix(as.vector(c(0.85,-0.2, 0.48,0,0.52,0.31,-0.79,0,0.01,0.93,0.36,0,0,0,0,1)), ncol=4, nrow=4)
# windowRect<-par3d()$windowRect # suggestion: windowRect = c(0, 64, 1920, 1111)

ooo = centroids[centroids$topCall=='',1:3]
xlim = c (range(scores[,1])[1]-9, range(scores[,1])[2]+7)
ylim = c (range(scores[,2])[1]-8, range(scores[,2])[2]+5)
zlim = c (range(scores[,3])[1]-.5, range(scores[,3])[2]+2)

myCols = as.character(key$allColors[match(mycalls, key$allCalls)])
# palette (rainbow(length(levels(dfTemp$topCall))))
palette (myCols)

open3d(zoom = zoom, userMatrix = userMatrix, windowRect=windowRect)
plot3d (scores[,1],scores[,2],scores[,3], box=F, axes=F, col=as.numeric(as.factor(dfTemp$topCall)), type='n', radius=.3, alpha=.3, size=5, xlab='', ylab='', zlab='', xlim=xlim, ylim=ylim, zlim=zlim) 
text3d (scores[,1],scores[,2],scores[,3], texts=dfTemp$topCall, col=as.numeric(as.factor(dfTemp$topCall)), size=1, alpha=1)
axis3d('x', pos=c(ooo[1],ooo[2], ooo[3]), labels=F, tick=F)
axis3d('y', pos=c(ooo[1],ooo[2], ooo[3]), labels=F, tick=F)
axis3d('z', pos=c(ooo[1],ooo[2], ooo[3]), labels=F, tick=F)
text3d(centroids$mds1, centroids$mds2, centroids$mds3, texts=mycalls, cex=2, col='black') 
text3d(centroids$mds1, centroids$mds2, centroids$mds3-.5, texts=centroids$topCall, cex=1.5, col='gray50') # except for English

snapshot3d(paste0('~/Downloads/MDS_',lang,'.png'))
# movie3d( spin3d(), duration=12, dir = '~/Downloads/' )  # make a movie. 12 gives a full 360° circle. 


