## How well does the chosen call type predict the chosen emotion type?
library (randomForest)
library (gplots)
library (entropy)

setwd (dir = '/home/allgoodguys/Documents/Studying/Lund_PhD/experiments/focal-terms/analysis')

lang = c('en','sv','ru')[3]

if (lang=='sv'){
  df1 = read.csv ('df1_sv.csv')[,-1]
  table_simpl = read.csv ('table_simpl_sv.csv')[,-1]
  calls = c('flämtning','fniss','gnäll','gråt','rop','skratt','skrik','snyftning','stön','suck','tjut','vrål','callDK')
  calls_en = c ('gasp','giggle','whimper','cry','yell','laugh','scream','sob','moan','sigh','howl','roar','DK')
  ems = c('ansträngning','frustration','glädje','hysteri','ilska','irritation','jubel','lättnad','munterhet','njutning','panik','rädsla','smärta','sorg','äckel','överraskning','emDK')
  ems_en = c('effort','frustration','happiness','hysteria','anger','irritation','triumph','relief','mirth','pleasure','panic','fear','pain','sadness','disgust','surprise','DK')
} else if (lang=='en') {
  df1 = read.csv ('df1_en.csv')[,-1]
  table_simpl = read.csv ('table_simpl_en.csv')[,-1]
  calls = c('chuckle','cry','gasp','giggle','groan','growl','grunt','laugh','moan','roar','scream','shriek','sigh','snort','sob','yell','callDK')
  ems = c('amusement','anger','annoyance','disgust','effort','excitement','fear','fright','frustration','happiness','hysteria','pain','pleasure','relief','sadness','surprise','emDK')
} else if (lang=='ru'){
  df1 = read.csv ('df1_ru.csv')[,-1]
  table_simpl = read.csv ('table_simpl_ru.csv')[,-1]
  calls = c('вздох','визг','вой','вопль','крик','кряхтенье','нытье','плач','рев','смех','стон','хихиканье','callDK')
  calls_en = c('sigh', 'shriek', 'howl', 'yell', 'scream', 'grunt', 'whimper','cry', 'roar', 'laugh', 'moan', 'giggle','DK') 
  ems = c('боль','веселье','восторг','грусть','досада','злость','испуг','истерика','облегчение','отвращение','радость','раздражение','страх','удивление','удовольствие','усилие','emDK')
  ems_en = c('pain','mirth','delight','sadness','annoyance','anger','fright','hysteria','relief','disgust','joy','irritation','fear','surprise','pleasure','effort','DK')
}
df1$nSoundNames = apply(df1[,c(calls)], 1, sum) # summary(df1$nSoundNames)
df1$nEmNames = apply(df1[,c(ems)], 1, sum) # summary(df1$nEmNames)

if (F){
  # to get a contingency table of call-emotion based on individual answers:
  topCall = apply (df1[,12:24], 1, function(x) names(which.max(x)))
  topEm = apply (df1[,25:ncol(df1)], 1, function(x) names(which.max(x)))
  write.csv (table(topCall, topEm), '~/Downloads/temp2.csv')
}

# or (same result):
a = as.matrix (df1[,c(calls)])
a = t(a)
b = as.matrix(df1[,c(ems)])
b = t(b)
# write.csv (a %*% t(b), '~/Downloads/temp.csv')

# make a continguency table with colored cells
mytable = a %*% t(b)
if (! lang=='en'){
  rownames(mytable) = paste0 (calls, ' (', calls_en, ')')
  colnames(mytable) = paste0 (ems, ' (', ems_en, ')')
}
rownames(mytable)[nrow(mytable)] = "Don't know"
colnames(mytable)[ncol(mytable)] = "Don't know"

mydist = function(mytable){
  # we normalize each row and return the dist. Why? B/c heatmap uses dist() as the default function to reorganize the dendrogram, but in our case absolute counts differ a lot (some words are used much more seldom), obscuring synonimity. After normalization the dendrogram seems to make more sense
  temp = t ( apply (mytable, 1, function(x)x/max(x)) )
  return (dist(temp))
}

filename = paste0('/home/allgoodguys/Documents/Studying/Lund_PhD/epistles/03_article_labels/pix/call-em-table_', lang, '.png')
png(file=filename, width=900, height=900, units='px')
heatmap.2 (x=mytable, Rowv=T, Colv=T, dendrogram="both", cexRow=2, cexCol=2, margins=c(20,18), notecol="black", notecex=1.5, cellnote=mytable, trace="none", breaks=11, key=F, col=c('white','gray95','gray90','gray85','gray80','gray75','gray70','gray60','gray50','gray40'), distfun=mydist)
text(x=-8,y=-22,labels='Emotion', cex=3)
dev.off()


# get chi-square and mutual information to summarize the association between sound names and emotion names
chisq.test(mytable)
mi.plugin(mytable) # mutual info (0=indep). 0.58 for en, 0.49 for sv, 0.49 for ru

# illustration of mi
lc = length(calls)
le = length(ems)
a = matrix(rep(1,le*lc),nrow=lc)
b = a; diag(b) = 1e16
c = matrix(rep(0,le*lc),nrow=lc); diag(c)=10

mi.plugin(a)
mi.plugin(b)
mi.plugin(c)

chisq.test(a)
chisq.test(b)
chisq.test(c)



### RF classification
table_simpl = droplevels(table_simpl)
mod.rf = randomForest (topEm~topCall, table_simpl) # using only the top call
mod.rf

formula = as.formula(paste("topEm ~ ", paste(calls, collapse=' + ')))
mod.rf = randomForest (formula, data=table_simpl ) # using all 12 (16) calls
mod.rf  # 67% accuracy for en, 60% for sv, 67% for ru


## Optional: analyze how often multiple sound vs emotion names were chosen by 1 participant for 1 sound. Basically: a bit > for sv&ru, about the same for en
require(brms)
require(shinystan)
require(reshape2)
require(lme4)

df = df1[,c('id','sound','nSoundNames','nEmNames')]
df = melt(df)

mod_p = glmer (value~variable + (1|sound) + (1|id), data = df, family='poisson')
summary (mod_p)

modBRMS_cat = brm (value~variable + (1|sound) + (1|id), data = df, n.warmup=100, n.iter=1000, n.chains=3, n.cluster=3, family='poisson')
# stancode (modBRMS_cat)
# saveRDS (modBRMS_cat, 'sure_stan_mod.RDS')

plot(modBRMS_cat)
summary(modBRMS_cat)

